import 'package:flutter/material.dart';
import 'package:cached_network_image/cached_network_image.dart';
import 'package:f0ckapp/models/MediaItem.dart';
import 'package:f0ckapp/services/Api.dart';
import 'package:f0ckapp/widgets/VideoWidget.dart';
import 'package:f0ckapp/utils/SmartRefreshIndicator.dart';
import 'package:f0ckapp/utils/PageTransformer.dart';
import 'package:flutter_cache_manager/flutter_cache_manager.dart';

class DetailView extends StatefulWidget {
  final int initialItemId;
  final List<MediaItem> mediaItems;
  final String type;
  final int mode;
  final bool random;
  final String? tagname;

  const DetailView({
    super.key,
    required this.initialItemId,
    required this.mediaItems,
    required this.type,
    required this.mode,
    required this.random,
    required this.tagname,
  });

  @override
  State createState() => _DetailViewState();
}

class _DetailViewState extends State<DetailView> {
  late PageController _pageController;
  late List<MediaItem> mediaItems;
  String? _tagname;
  int currentItemId = 0;
  bool isLoading = false;

  @override
  void initState() {
    super.initState();
    mediaItems = widget.mediaItems;
    _tagname = widget.tagname;
    final initialIndex = mediaItems.indexWhere(
      (item) => item.id == widget.initialItemId,
    );
    _pageController = PageController(initialPage: initialIndex);
    currentItemId = mediaItems[initialIndex].id;

    int? lastLoadedIndex;
    _pageController.addListener(() async {
      final newIndex = _pageController.page?.round();
      if (newIndex != null &&
          newIndex < mediaItems.length &&
          newIndex != lastLoadedIndex) {
        setState(() => currentItemId = mediaItems[newIndex].id);
        lastLoadedIndex = newIndex;

        _preloadAdjacentMedia(newIndex);
      }

      if (_pageController.position.pixels >=
          _pageController.position.maxScrollExtent - 100) {
        _loadMoreMedia();
      }
    });

    _preloadAdjacentMedia(initialIndex);
  }

  void _preloadAdjacentMedia(int index) async {
    if (index + 1 < mediaItems.length) {
      final nextUrl = mediaItems[index + 1].mediaUrl;
      if (await DefaultCacheManager().getFileFromCache(nextUrl) == null) {
        await DefaultCacheManager().downloadFile(nextUrl);
        print('preload ${mediaItems[index + 1].id}');
      }
    }

    if (index - 1 >= 0) {
      final prevUrl = mediaItems[index - 1].mediaUrl;
      if (await DefaultCacheManager().getFileFromCache(prevUrl) == null) {
        await DefaultCacheManager().downloadFile(prevUrl);
        print('preload ${mediaItems[index - 1].id}');
      }
    }
  }

  Future<void> _loadMoreMedia() async {
    if (isLoading) return;
    setState(() => isLoading = true);

    try {
      final newMedia = await fetchMedia(
        older: mediaItems.last.id.toString(),
        type: widget.type,
        mode: widget.mode,
        random: widget.random,
        tag: _tagname,
      );
      if (mounted && newMedia.isNotEmpty) {
        setState(() {
          mediaItems.addAll(newMedia);
          isLoading = false;
        });
      }
    } catch (e) {
      _showError("Ein unerwarteter Fehler ist aufgetreten: $e");
    }
  }

  Future<void> _refreshMediaItem() async {
    try {
      final updatedItem = await fetchMediaDetail(currentItemId);
      if (mounted) {
        final index = mediaItems.indexWhere((item) => item.id == currentItemId);
        if (index != -1) {
          setState(() => mediaItems[index] = updatedItem);
        }
      }
    } catch (e) {
      _showError("Fehler beim Aktualisieren des Items: $e");
    }
  }

  void _showError(String message) {
    if (!mounted) return;

    final messenger = ScaffoldMessenger.of(context);
    messenger.hideCurrentSnackBar();
    messenger.showSnackBar(SnackBar(content: Text(message)));
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: const Color(0xFF171717),
      appBar: AppBar(
        backgroundColor: const Color(0xFF2B2B2B),
        foregroundColor: Colors.white,
        title: Text('f0ck #$currentItemId (${widget.type})'),
        centerTitle: true,
      ),
      body: Stack(
        children: [
          PageTransformer(
            controller: _pageController,
            pages: mediaItems.map((item) {
              final isActive = item.id == currentItemId;
              return Scaffold(
                body: SafeArea(
                  child: SmartRefreshIndicator(
                    onRefresh: _refreshMediaItem,
                    child: _buildMediaItem(item, isActive),
                  ),
                ),
              );
            }).toList(),
          ),
          if (_tagname != null)
            Positioned(
              bottom: 60,
              left: MediaQuery.of(context).size.width * 0.2,
              right: MediaQuery.of(context).size.width * 0.2,
              child: Container(
                padding: const EdgeInsets.symmetric(
                  vertical: 10,
                  horizontal: 20,
                ),
                decoration: BoxDecoration(
                  color: Colors.black.withValues(alpha: 0.8),
                  borderRadius: BorderRadius.circular(12),
                ),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: [
                    Text(
                      'Tag: $_tagname',
                      style: const TextStyle(color: Colors.white),
                    ),
                    IconButton(
                      icon: const Icon(Icons.close, color: Colors.white),
                      onPressed: () {
                        setState(() {
                          _tagname = '___empty___';
                          Navigator.pop(context, _tagname);
                        });
                      },
                    ),
                  ],
                ),
              ),
            ),
        ],
      ),
    );
  }

  Widget _buildMediaItem(MediaItem item, bool isActive) {
    return SingleChildScrollView(
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          if (item.mime.startsWith('image'))
            CachedNetworkImage(
              imageUrl: item.mediaUrl,
              fit: BoxFit.contain,
              placeholder: (context, url) => CircularProgressIndicator(),
              errorWidget: (context, url, error) => Icon(Icons.error),
            )
          else
            VideoWidget(details: item, isActive: isActive),
          const SizedBox(height: 20),
          Text(
            item.mime,
            style: const TextStyle(color: Colors.white, fontSize: 18),
          ),
          const SizedBox(height: 10, width: double.infinity),
          Wrap(
            alignment: WrapAlignment.center,
            spacing: 5.0,
            children: item.tags.map((tag) {
              return ActionChip(
                label: Text(
                  tag.tag,
                  style: const TextStyle(
                    color: Colors.white,
                    fontWeight: FontWeight.bold,
                  ),
                ),
                backgroundColor: switch (tag.id) {
                  1 => Colors.green,
                  2 => Colors.red,
                  _ => const Color(0xFF090909),
                },
                padding: const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
                shape: RoundedRectangleBorder(
                  borderRadius: BorderRadius.circular(12),
                ),
                onPressed: () async {
                  if (tag.tag == 'sfw' || tag.tag == 'nsfw') return;
                  setState(() => Navigator.pop(context, tag.tag));
                },
              );
            }).toList(),
          ),
          const SizedBox(height: 20),
        ],
      ),
    );
  }
}
